#include <QApplication>
#include <QTreeView>
#include "remotetablemodel.h"
#include <QList>
#include <QBuffer>
#include <QHttp>
#include <QRegExp>

class MyUpdater : public Updater {
  Q_OBJECT
public:
  MyUpdater() : Updater(){
    _p = 1;
    connect(&http, SIGNAL(readyRead( const QHttpResponseHeader &)), SLOT(read()));
    connect(&http, SIGNAL(requestFinished(int, bool)), SLOT(onFinished(int)));
  }
protected:
  void retrieveAll(){
    buff.open(QBuffer::WriteOnly);
    http.setHost("www.qtcentre.org", 80);
    http.get("/forum/archive/index.php/f-2.html");    
  }
  QHttp http;
  QBuffer buff;
  int _p;
private slots:
  void onFinished(int id){
    if(id==1 || _p>15) return;
    http.get(QString("/forum/archive/index.php/f-2-p-%1.html").arg(++_p));
  }
  void read(){
        buff.write(http.readAll());
        QBuffer readBuffer(&buff.buffer());
        readBuffer.open(QBuffer::ReadOnly);
        QRegExp rx("<li><a href=\"t-(\\d+)\\.html\">(.*)</a></li>");
        while(readBuffer.canReadLine()){
          QString line = readBuffer.readLine();
          if(rx.indexIn(line)>=0){
            QString name = rx.cap(2);
            int tid = rx.cap(1).toInt();
            QVariantList vlist;
            vlist << name << tid;
            emit retrieved(QVariant(vlist));
          }
        }
  }
};

class MyUpdaterFactory : public UpdaterFactory {
  Updater *create(){
    return new MyUpdater();
  }
};

#include "main.moc"

class MyModel : public RemoteTableModel {
public:
  MyModel() : RemoteTableModel(){
    setUpdaterFactory(new MyUpdaterFactory());
    start();
  }
  int columnCount ( const QModelIndex & parent = QModelIndex() ) const{
    if(parent.isValid()) return 0;
    return 2;
  }
  int rowCount( const QModelIndex &parent = QModelIndex() ) const {
    if(parent.isValid()) return 0;
    return m_rows.count();
  }
  QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const{
    if(!index.isValid() || role!=Qt::DisplayRole) return QVariant();
    int row = index.row();
    if(row>=m_rows.count()) return QVariant();
    if(index.column()==1){
      return m_rows.at(row).title;
    } else {
      return m_rows.at(row).tid;    
    }
  }
protected:
  void addRow(const QVariant &data){
    beginInsertRows(QModelIndex(), m_rows.size(), m_rows.size());
    QVariantList vlist = data.toList();
    m_rows << st(vlist.at(0).toString(), vlist.at(1).toInt());
    endInsertRows();
  }
private:
  struct st {
    st(QString t, int i){ title = t; tid = i; }
    QString title;
    int tid;
  };
  QList<st> m_rows;
};


int main(int argc, char **argv){
  QApplication app(argc, argv);
  MyModel model;
  QTreeView tv;
  tv.setRootIsDecorated(false);
  tv.setModel(&model);
  tv.show();
  return app.exec();
}
