#ifndef __REMOTETABLEMODEL_H
#define __REMOTETABLEMODEL_H

#include <QAbstractTableModel>
class RemoteTableModelPrivate;


/**
 *  Class responsible for updating the remote storage. 
 *  Reimplement to fit your needs
 */
class Updater : public QObject {
  Q_OBJECT
public:
  Updater() : QObject(){}
signals:
  void retrieved(const QVariant &data);
protected:
  virtual void retrieveAll();
  virtual void retrieveRow(int rowid);
  virtual void setData(int rowid, int column, const QVariant &, int role=Qt::EditRole);
  virtual void removeRow(int rowid);
  virtual void insertRow(int rowid);
  void customEvent ( QEvent * event );
};

/**
 *  A terrible hack - needed to create instances of Updater subclasses
 *
 */
class UpdaterFactory {
public:
  UpdaterFactory(){}
  virtual Updater *create()=0;
  virtual ~UpdaterFactory(){}
};

/**
 *  Remote model
 *
 *
 */
class RemoteTableModel : public QAbstractTableModel {
  Q_OBJECT
public:
  RemoteTableModel(QObject *parent = 0);
public slots:
  void start();
protected slots:
  virtual void addRow(const QVariant &data)=0;
protected:
  void remoteSetData(int rowid, int column, const QVariant &data, int role=Qt::EditRole);
  void remoteInsertRow(int rowid);
  void remoteRemoveRow(int rowid);
  void setUpdaterFactory(UpdaterFactory *f);
private:
  RemoteTableModelPrivate *d() const { return m_d; }
  RemoteTableModelPrivate *m_d;
private slots:
  void beginFill();
};

#endif
